﻿using Dapper;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;

namespace DotNetNote.Models
{
    public class NoteRepository : INoteRepository
    {
        private SqlConnection con;
        private ILogger<NoteRepository> _logger;
        private IConfiguration _config;

        public NoteRepository(IConfiguration config, ILogger<NoteRepository> logger)
        {
            _config = config; 
            con = new SqlConnection(
                _config.GetSection("Data").GetSection("DefaultConnection").GetSection("ConnectionString").Value);
            _logger = logger;
        }
        
        public void Add(Note model)
        {
            _logger.LogInformation("데이터 입력");
            try
            {
                // 파라미터 추가
                var parameters = new DynamicParameters();
                parameters.Add("@Name", value: model.Name, dbType: DbType.String);
                parameters.Add("@Email", value: model.Email, dbType: DbType.String);
                parameters.Add("@Title", value: model.Title, dbType: DbType.String);
                parameters.Add("@PostIp", value: model.PostIp, dbType: DbType.String);
                parameters.Add("@Content", value: model.Content, dbType: DbType.String);
                parameters.Add("@Password", value: model.Password, dbType: DbType.String);
                parameters.Add("@Encoding", value: model.Encoding, dbType: DbType.String);
                parameters.Add("@Homepage", value: model.Homepage, dbType: DbType.String);
                parameters.Add("@FileName", value: model.FileName, dbType: DbType.String);
                parameters.Add("@FileSize", value: model.FileSize, dbType: DbType.Int32);

                con.Execute("WriteNote", parameters, commandType: CommandType.StoredProcedure);
            }
            catch (System.Exception ex)
            {
                _logger.LogError("데이터 입력 에러: " + ex);
            }
        }

        public List<Note> GetAll(int page)
        {
            _logger.LogInformation("데이터 출력");
            try
            {
                var parameters = new DynamicParameters(new { Page = page });
                return con.Query<Note>("ListNotes", parameters, commandType: CommandType.StoredProcedure).ToList();
            }
            catch (System.Exception ex)
            {
                _logger.LogError("데이터 출력 에러: " + ex);
                return null; 
            }
        }

        /// <summary>
        /// 검색 카운트
        /// </summary>
        public int GetCountBySearch(string searchField, string searchQuery)
        {
            try
            {
                return con.Query<int>("SearchNoteCount", new { SearchField = searchField, SearchQuery = searchQuery }, commandType: CommandType.StoredProcedure).SingleOrDefault();

            }
            catch (System.Exception ex)
            {
                _logger.LogError("카운트 출력 에러: " + ex);
                return -1; 
            }
        }

        /// <summary>
        /// Notes 테이블의 모든 레코드 수
        /// </summary>
        public int GetCount()
        {
            try
            {
                return con.Query<int>("Select Count(*) From Notes").SingleOrDefault();
            }
            catch (System.Exception)
            {
                _logger.LogError("에러 발생");
                return -1;
            }
        }

        /// <summary>
        /// Id에 해당하는 파일명 반환
        /// </summary>
        public string GetFileNameById(int id)
        {
            return con.Query<string>("Select FileName From Notes Where Id = @Id", new { Id = id }).SingleOrDefault();
        }

        /// <summary>
        /// 검색 결과 리스트
        /// </summary>
        public List<Note> GetSeachAll(int page, string searchField, string searchQuery)
        {
            var parameters = new DynamicParameters(new { Page = page, SearchField = searchField, SearchQuery = searchQuery });
            return con.Query<Note>("SearchNotes", parameters, commandType: CommandType.StoredProcedure).ToList();
        }

        /// <summary>
        /// 다운 카운트 1 증가
        /// </summary>
        public void UpdateDownCount(string fileName)
        {
            con.Execute("Update Notes Set DownCount = DownCount + 1 Where FileName = @FileName", new { FileName = fileName });
        }

        /// <summary>
        /// 상세 보기 
        /// </summary>
        public Note GetNoteById(int id)
        {
            var parameters = new DynamicParameters(new { Id = id });
            return con.Query<Note>("ViewNote", parameters, commandType: CommandType.StoredProcedure).SingleOrDefault();
        }


        /// <summary>
        /// 삭제 
        /// </summary>
        public int DeleteNote(int id, string password)
        {
            return con.Execute("DeleteNote", new { Id = id, Password = password }, commandType: CommandType.StoredProcedure); 
        }


        /// <summary>
        /// 수정하기
        /// </summary>
        public int UpdateNote(Note model)
        {
            // 파라미터 추가
            var parameters = new DynamicParameters();
            parameters.Add("@Name", value: model.Name, dbType: DbType.String);
            parameters.Add("@Email", value: model.Email, dbType: DbType.String);
            parameters.Add("@Title", value: model.Title, dbType: DbType.String);
            parameters.Add("@ModifyIp", value: model.ModifyIp, dbType: DbType.String);
            parameters.Add("@Content", value: model.Content, dbType: DbType.String);
            parameters.Add("@Password", value: model.Password, dbType: DbType.String);
            parameters.Add("@Encoding", value: model.Encoding, dbType: DbType.String);
            parameters.Add("@Homepage", value: model.Homepage, dbType: DbType.String);
            parameters.Add("@FileName", value: model.FileName, dbType: DbType.String);
            parameters.Add("@FileSize", value: model.FileSize, dbType: DbType.Int32);
            parameters.Add("@Id", value: model.Id, dbType: DbType.Int32);

            return con.Execute("ModifyNote", parameters, commandType: CommandType.StoredProcedure);
        }

        /// <summary>
        /// 답변 글쓰기
        /// </summary>
        public void ReplyNote(Note model)
        {
            // 파라미터 추가
            var parameters = new DynamicParameters();
            parameters.Add("@Name", value: model.Name, dbType: DbType.String);
            parameters.Add("@Email", value: model.Email, dbType: DbType.String);
            parameters.Add("@Title", value: model.Title, dbType: DbType.String);
            parameters.Add("@PostIp", value: model.PostIp, dbType: DbType.String);
            parameters.Add("@Content", value: model.Content, dbType: DbType.String);
            parameters.Add("@Password", value: model.Password, dbType: DbType.String);
            parameters.Add("@Encoding", value: model.Encoding, dbType: DbType.String);
            parameters.Add("@Homepage", value: model.Homepage, dbType: DbType.String);
            parameters.Add("@FileName", value: model.FileName, dbType: DbType.String);
            parameters.Add("@FileSize", value: model.FileSize, dbType: DbType.Int32);
            parameters.Add("@ParentNum", value: model.ParentNum, dbType: DbType.Int32);

            con.Execute("ReplyNote", parameters, commandType: CommandType.StoredProcedure);
        }

    }
}
