﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNet.Builder;
using Microsoft.AspNet.Hosting;
using Microsoft.AspNet.Identity.EntityFramework;
using Microsoft.Data.Entity;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using DotNetNote.Models;
using DotNetNote.Services;
using MemoEngine.Repositories;
using MemoEngine.Controllers;
using Newtonsoft.Json.Serialization;

namespace DotNetNote
{
    public class Startup
    {
        public Startup(IHostingEnvironment env)
        {
            // Set up configuration sources.
            var builder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true);

            if (env.IsDevelopment())
            {
                // For more details on using the user secret store see http://go.microsoft.com/fwlink/?LinkID=532709
                builder.AddUserSecrets();
            }

            builder.AddEnvironmentVariables();
            Configuration = builder.Build();
        }

        public IConfigurationRoot Configuration { get; set; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            // Add framework services.
            services.AddEntityFramework()
                .AddSqlServer()
                .AddDbContext<ApplicationDbContext>(options =>
                    options.UseSqlServer(Configuration["Data:DefaultConnection:ConnectionString"]));

            services.AddIdentity<ApplicationUser, IdentityRole>()
                .AddEntityFrameworkStores<ApplicationDbContext>()
                .AddDefaultTokenProviders();

            services.AddCaching();
            services.AddSession();

            services.AddMvc()
                      .AddJsonOptions(opt =>
                      {
                          opt.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
                      }); ;



            // Add application services.
            services.AddTransient<IEmailSender, AuthMessageSender>();
            services.AddTransient<ISmsSender, AuthMessageSender>();



            //[!] Configuration 개체 주입
            services.AddInstance<IConfiguration>(Configuration);



            //[1] 게시판 관련 서비스 등록
            //[1] 생성자에 문자열로 연결 문자열 지정
            //services.AddInstance<INoteRepository>(new NoteRepository(Configuration["Data:DefaultConnection:ConnectionString"]));
            //[2] 의존성 주입으로 Configuration 주입
            services.AddTransient<INoteRepository, NoteRepository>(); 
            services.AddInstance<INoteCommentRepository>(new NoteCommentRepository(Configuration["Data:DefaultConnection:ConnectionString"]));

            services.AddTransient<MaximServiceRepository, MaximServiceRepository>();
            services.AddTransient<MaximController, MaximController>();


            // 서비스 등록
            // services.AddInstance<CopyrightService>(new CopyrightService());
            //services.AddInstance<ICopyrightService>(new CopyrightService()); // 동일, 새로 고침 X
            //services.AddSingleton<ICopyrightService, CopyrightService>(); // 동일, 새로 고침 X
            services.AddTransient<ICopyrightService, CopyrightService>(); // 매번 다름, 새로 고침 적용
            //services.AddScoped<ICopyrightService, CopyrightService>(); // 같은 요청(Request)일 때 동일, 새로 고침 적용
                        
            // @inject 키워드로 뷰에 직접 클래스의 속성 또는 메서드 값 출력
            services.AddSingleton<CopyrightService>(); 


        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env, ILoggerFactory loggerFactory)
        {
            loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            loggerFactory.AddDebug();

            if (env.IsDevelopment())
            {
                app.UseBrowserLink();
                app.UseDeveloperExceptionPage();
                app.UseDatabaseErrorPage();
            }
            else
            {
                app.UseExceptionHandler("/Home/Error");

                // For more details on creating database during deployment see http://go.microsoft.com/fwlink/?LinkID=615859
                try
                {
                    using (var serviceScope = app.ApplicationServices.GetRequiredService<IServiceScopeFactory>()
                        .CreateScope())
                    {
                        serviceScope.ServiceProvider.GetService<ApplicationDbContext>()
                             .Database.Migrate();
                    }
                }
                catch { }
            }

            app.UseIISPlatformHandler(options => options.AuthenticationDescriptions.Clear());

            app.UseStaticFiles();

            app.UseIdentity();
            // To configure external authentication please see http://go.microsoft.com/fwlink/?LinkID=532715

            app.UseSession();

            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=DotNetNote}/{action=Index}/{id?}");
            });
        }

        // Entry point for the application.
        public static void Main(string[] args) => WebApplication.Run<Startup>(args);
    }
}
